// update() 和 UseWorker()分别对应着同步计算和异步计算的代码
// 将update()UseWorker()b表示使用异步计算
// 反之，表示使用同步计算

import 'libs/weapp-adapter.js';

let THREE = require('./libs/Three')
import 'libs/TrackballControls.js';
import CANNON from './cannon.js';
var world;

var controls;
// Parameters
var dt = 1 / 15,
  N = 100;

// Data arrays. Contains all our kinematic data we need for rendering.
var positions = new Float32Array(N * 3);
var quaternions = new Float32Array(N * 4);

var sendTime; // Time when we sent last message
function UseWorker() {
  const worker = wx.createWorker('workers/index.js') //, {useExperimentalWorker: true});
  worker.onMessage(e=>{
    // Get fresh data from the worker
    positions = e.positions;
    quaternions = e.quaternions;
  
    // Update rendering meshes
    for (var i = 0; i !== meshes.length; i++) {
      meshes[i].position.set(positions[3 * i + 0],
        positions[3 * i + 1],
        positions[3 * i + 2]);
      meshes[i].quaternion.set(quaternions[4 * i + 0],
        quaternions[4 * i + 1],
        quaternions[4 * i + 2],
        quaternions[4 * i + 3]);
    }
    // If the worker was faster than the time step (dt seconds), we want to delay the next timestep
    var delay = dt * 1000 - (Date.now() - sendTime);
    if (delay < 0) {
      delay = 0;
    }
  
    // setTimeout(sendDataToWorker, delay);
  })
}


// function sendDataToWorker() {
//   sendTime = Date.now();
//   worker.postMessage({
//     N: N,
//     dt: dt
//   });
// }

var camera, scene, renderer;
var meshes = [];

init();
animate();
// Start the worker!
UseWorker();               // 异步计算
// update();               // 同步计算

function init() {
  // scene
  scene = new THREE.Scene();
  scene.fog = new THREE.Fog(0x000000, 500, 10000);

  // camera
  camera = new THREE.PerspectiveCamera(45, window.innerWidth / window.innerHeight, 1, 10000);
  camera.position.set(60, 50, 60);
  camera.lookAt(scene.position);
  scene.add(camera);

  // Controls
  controls = new THREE.TrackballControls(camera);
  controls.rotateSpeed = 1.0;
  controls.zoomSpeed = 1.2;
  controls.panSpeed = 0.8;
  controls.noZoom = false;
  controls.noPan = false;
  controls.staticMoving = true;
  controls.dynamicDampingFactor = 0.3;
  controls.keys = [65, 83, 68];

  // lights
  var light, materials;
  scene.add(new THREE.AmbientLight(0x666666));

  light = new THREE.DirectionalLight(0xffffff, 1.75);
  var d = 20;

  light.position.set(d, d, d);

  light.castShadow = false;
  //light.shadowCameraVisible = true;

  light.shadowMapWidth = 1024;
  light.shadowMapHeight = 1024;

  light.shadowCameraLeft = -d;
  light.shadowCameraRight = d;
  light.shadowCameraTop = d;
  light.shadowCameraBottom = -d;

  light.shadowCameraFar = 3 * d;
  light.shadowCameraNear = d;
  light.shadowDarkness = 0.5;

  scene.add(light);

  // floor
  var geometry = new THREE.PlaneGeometry(75, 75, 1, 1);
  //geometry.applyMatrix( new THREE.Matrix4().makeRotationX( Math.PI / 2 ) );
  var material = new THREE.MeshLambertMaterial({
    color: 0x777777
  });
  //THREE.ColorUtils.adjustHSV( material.color, 0, 0, 0.9 );
  var mesh = new THREE.Mesh(geometry, material, 0, 50, 50);
  mesh.position.set(screen.width/2, screen.height /2, 0);
  mesh.castShadow = true;
  mesh.receiveShadow = true;
  mesh.rotation.x = Math.PI / 2;
  
  meshes.push(mesh);
  scene.add(mesh);
  // cubes
  var cubeGeo = new THREE.BoxGeometry(1, 1, 1, 1, 1,1);
  var cubeMaterial = new THREE.MeshPhongMaterial({
    color: 0x888888
  });
  for (var i = 0; i < N; i++) {
    var cubeMesh = new THREE.Mesh(cubeGeo, cubeMaterial);
    cubeMesh.castShadow = true;
    meshes.push(cubeMesh);
    scene.add(cubeMesh);
  }
  let ctx = canvas.getContext('webgl');
  renderer = new THREE.WebGLRenderer({
    canvas: canvas,
    context: ctx ,
    antialias: true,
  });
  renderer.setSize(window.innerWidth, window.innerHeight);
  renderer.setClearColor(scene.fog.color);

  renderer.gammaInput = true;
  renderer.gammaOutput = true;
  renderer.shadowMapEnabled = true;

  window.addEventListener('resize', onWindowResize, false);
}

function onWindowResize() {
  camera.aspect = window.innerWidth / window.innerHeight;
  camera.updateProjectionMatrix();
  controls.handleResize();
  renderer.setSize(window.innerWidth, window.innerHeight);
}

function animate() {
  requestAnimationFrame(animate);
  controls.update();
  render();
}

function render() {
  renderer.render(scene, camera);
}
 
function update() {
  sendTime = Date.now()
  if (!world) {
    // Init physics
    world = new CANNON.World();
    world.broadphase = new CANNON.NaiveBroadphase();
    world.gravity.set(0, -10, 0);
    world.solver.tolerance = 0.001;

    // Ground plane
    var plane = new CANNON.Plane();
    var groundBody = new CANNON.Body({
        mass: 0
    });
    groundBody.addShape(plane);
    groundBody.quaternion.setFromAxisAngle(new CANNON.Vec3(1, 0, 0), -Math.PI / 2);
    world.add(groundBody);

    // Create N cubes
    var shape = new CANNON.Box(new CANNON.Vec3(0.5, 0.5, 0.5));
    for (var i = 0; i !== N; i++) {
        var body = new CANNON.Body({
            mass: 1
        });
        body.addShape(shape);
        body.position.set(Math.random() - 0.5, 2.5 * i + 0.5, Math.random() - 0.5);
        world.add(body);
    }
  }
  world.step(dt);
  for (var i = 0; i !== world.bodies.length; i++) {
        var b = world.bodies[i],
        p = b.position,
        q = b.quaternion;
        meshes[i].position.set(p.x, p.y, p.z);
        meshes[i].quaternion.set(q.x,q.y,q.z,q.w);
  }
    // If the worker was faster than the time step (dt seconds), we want to delay the next timestep
    console.log("total compute:", Date.now() - sendTime)
    var delay = dt * 1000 - (Date.now() - sendTime);
    if (delay < 0) {
      delay = 0;
    }
  
    setTimeout(update, delay);
}