import CANNON from './cannon.js';
var world;
var dt = 1 / 15,
N = 100;


update()
  function update() {
    var sendTime = Date.now()
    if (!world) {
      // Init physics
      world = new CANNON.World();
      world.broadphase = new CANNON.NaiveBroadphase();
      world.gravity.set(0, -10, 0);
      world.solver.tolerance = 0.001;
  
      // Ground plane
      var plane = new CANNON.Plane();
      var groundBody = new CANNON.Body({
          mass: 0
      });
      groundBody.addShape(plane);
      groundBody.quaternion.setFromAxisAngle(new CANNON.Vec3(1, 0, 0), -Math.PI / 2);
      world.add(groundBody);
  
      // Create N cubes
      var shape = new CANNON.Box(new CANNON.Vec3(0.5, 0.5, 0.5));
      for (var i = 0; i !== N; i++) {
          var body = new CANNON.Body({
              mass: 1
          });
          body.addShape(shape);
          body.position.set(Math.random() - 0.5, 2.5 * i + 0.5, Math.random() - 0.5);
          world.add(body);
      }
    }
    world.step(dt);
    var positions = new Float32Array(N * 3);
    var quaternions = new Float32Array(N * 4);
    for (var i = 0; i !== world.bodies.length; i++) {
        var b = world.bodies[i],
            p = b.position,
            q = b.quaternion;
        positions[3 * i + 0] = p.x;
        positions[3 * i + 1] = p.y;
        positions[3 * i + 2] = p.z;
        quaternions[4 * i + 0] = q.x;
        quaternions[4 * i + 1] = q.y;
        quaternions[4 * i + 2] = q.z;
        quaternions[4 * i + 3] = q.w;
    }
    worker.postMessage({
        positions: positions,
        quaternions: quaternions
      });
      // If the worker was faster than the time step (dt seconds), we want to delay the next timestep
      console.log("total compute:", Date.now() - sendTime)
      var delay = dt * 1000 - (Date.now() - sendTime);
      if (delay < 0) {
        delay = 0;
      }
      
      setTimeout(update, delay);
  }
